@@ -20,16 +20,15 @@ import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 
-from neutron.db import _model_query as model_query
-from neutron.db import api as db_api
-from neutron.db import common_db_mixin
-from neutron.db import standard_attr
-
 from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
 from neutron_lib.api.definitions import bgpvpn_routes_control as bgpvpn_rc_def
 from neutron_lib.api.definitions import bgpvpn_vni as bgpvpn_vni_def
+from neutron_lib.db import api as db_api
 from neutron_lib.db import constants as db_const
 from neutron_lib.db import model_base
+from neutron_lib.db import model_query
+from neutron_lib.db import standard_attr
+from neutron_lib.db import utils as db_utils
 from neutron_lib.plugins import directory
 
 from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
@@ -118,8 +117,6 @@ class BGPVPNPortAssociation(standard_attr.HasStandardAttributes,
                             backref=orm.backref('bgpvpn_associations',
                                                 cascade='all'),
                             lazy='joined',)
-    routes = orm.relationship("BGPVPNPortAssociationRoute",
-                              backref="bgpvpn_port_associations")
 
     # standard attributes support:
     api_collections = []
@@ -128,47 +125,12 @@ class BGPVPNPortAssociation(standard_attr.HasStandardAttributes,
                                bgpvpn_rc_def.PORT_ASSOCIATION}
 
 
-class BGPVPNPortAssociationRoute(model_base.BASEV2, model_base.HasId):
-    """Represents an item of the 'routes' attribute of a port association."""
-    __tablename__ = 'bgpvpn_port_association_routes'
-
-    port_association_id = sa.Column(
-        sa.String(length=36),
-        sa.ForeignKey('bgpvpn_port_associations.id', ondelete='CASCADE'),
-        nullable=False)
-    type = sa.Column(sa.Enum(*bgpvpn_rc_def.ROUTE_TYPES,
-                             name="bgpvpn_port_assoc_route_type"),
-                     nullable=False)
-    local_pref = sa.Column(sa.BigInteger(),
-                           nullable=True)
-    # prefix is NULL unless type is 'prefix'
-    prefix = sa.Column(sa.String(49),
-                       nullable=True)
-    # bgpvpn_id is NULL unless type is 'bgpvpn'
-    bgpvpn_id = sa.Column(sa.String(length=36),
-                          sa.ForeignKey('bgpvpns.id', ondelete='CASCADE'),
-                          nullable=True)
-
-    port_association = orm.relationship(
-        "BGPVPNPortAssociation",
-        backref=orm.backref('port_association_routes',
-                            cascade='all'),
-        lazy='joined')
-    bgpvpn = orm.relationship(
-        "BGPVPN",
-        backref=orm.backref("port_association_routes",
-                            uselist=False,
-                            lazy='select',
-                            cascade='all, delete-orphan'),
-        lazy='joined')
-
-
 class BGPVPN(standard_attr.HasStandardAttributes, model_base.BASEV2,
              model_base.HasId, model_base.HasProject):
     """Represents a BGPVPN Object."""
     name = sa.Column(sa.String(255))
     type = sa.Column(sa.Enum("l2", "l3",
-                             name="bgpvpn_type"),
+                             name="vpn_types"),
                      nullable=False)
     route_targets = sa.Column(sa.String(255), nullable=False)
     import_targets = sa.Column(sa.String(255), nullable=True)
@@ -195,6 +157,41 @@ class BGPVPN(standard_attr.HasStandardAttributes, model_base.BASEV2,
                                bgpvpn_def.RESOURCE_NAME}
 
 
+class BGPVPNPortAssociationRoute(model_base.BASEV2, model_base.HasId):
+    """Represents an item of the 'routes' attribute of a port association."""
+    __tablename__ = 'bgpvpn_port_association_routes'
+
+    port_association_id = sa.Column(
+        sa.String(length=36),
+        sa.ForeignKey('bgpvpn_port_associations.id', ondelete='CASCADE'),
+        nullable=False)
+    type = sa.Column(sa.Enum(*bgpvpn_rc_def.ROUTE_TYPES,
+                             name="bgpvpn_port_association_route_type"),
+                     nullable=False)
+    local_pref = sa.Column(sa.BigInteger(),
+                           nullable=True)
+    # prefix is NULL unless type is 'prefix'
+    prefix = sa.Column(sa.String(49),
+                       nullable=True)
+    # bgpvpn_id is NULL unless type is 'bgpvpn'
+    bgpvpn_id = sa.Column(sa.String(length=36),
+                          sa.ForeignKey('bgpvpns.id', ondelete='CASCADE'),
+                          nullable=True)
+
+    port_association = orm.relationship(
+        "BGPVPNPortAssociation",
+        backref=orm.backref('routes',
+                            cascade='all'),
+        lazy='joined')
+    bgpvpn = orm.relationship(
+        "BGPVPN",
+        backref=orm.backref("port_association_routes",
+                            uselist=False,
+                            lazy='select',
+                            cascade='all, delete-orphan'),
+        lazy='joined')
+
+
 def _list_bgpvpns_result_filter_hook(query, filters):
     values = filters and filters.get('networks', [])
     if values:
@@ -214,7 +211,7 @@ def _list_bgpvpns_result_filter_hook(query, filters):
     return query
 
 
-@db_api.context_manager.writer
+@db_api.CONTEXT_WRITER
 def _add_port_assoc_route_db_from_dict(context, route, port_association_id):
     if route['type'] == 'prefix':
         kwargs = {'prefix': route['prefix']}
@@ -244,7 +241,7 @@ def port_assoc_route_dict_from_db(route_db):
     return route
 
 
-class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
+class BGPVPNPluginDb():
     """BGPVPN service plugin database class using SQLAlchemy models."""
 
     def __new__(cls, *args, **kwargs):
@@ -256,7 +253,7 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             result_filters=_list_bgpvpns_result_filter_hook)
         return super(BGPVPNPluginDb, cls).__new__(cls, *args, **kwargs)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _get_bgpvpns_for_tenant(self, session, tenant_id, fields):
         try:
             qry = session.query(BGPVPN)
@@ -267,7 +264,7 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
         return [self._make_bgpvpn_dict(bgpvpn, fields=fields)
                 for bgpvpn in bgpvpns]
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _make_bgpvpn_dict(self, bgpvpn_db, fields=None):
         net_list = [net_assocs.network_id for net_assocs in
                     bgpvpn_db.network_associations]
@@ -300,16 +297,16 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             res[bgpvpn_rc_def.LOCAL_PREF_KEY] = bgpvpn_db.get(
                 bgpvpn_rc_def.LOCAL_PREF_KEY)
 
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def create_bgpvpn(self, context, bgpvpn):
         rt = utils.rtrd_list2str(bgpvpn['route_targets'])
         i_rt = utils.rtrd_list2str(bgpvpn['import_targets'])
         e_rt = utils.rtrd_list2str(bgpvpn['export_targets'])
         rd = utils.rtrd_list2str(bgpvpn.get('route_distinguishers', ''))
 
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             bgpvpn_db = BGPVPN(
                 id=uuidutils.generate_uuid(),
                 tenant_id=bgpvpn['tenant_id'],
@@ -326,24 +323,25 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
 
         return self._make_bgpvpn_dict(bgpvpn_db)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_bgpvpns(self, context, filters=None, fields=None):
-        return self._get_collection(context, BGPVPN, self._make_bgpvpn_dict,
-                                    filters=filters, fields=fields)
+        return model_query.get_collection(
+            context, BGPVPN, self._make_bgpvpn_dict,
+            filters=filters, fields=fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _get_bgpvpn(self, context, id):
         try:
-            return self._get_by_id(context, BGPVPN, id)
+            return model_query.get_by_id(context, BGPVPN, id)
         except exc.NoResultFound:
             raise bgpvpn_ext.BGPVPNNotFound(id=id)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_bgpvpn(self, context, id, fields=None):
         bgpvpn_db = self._get_bgpvpn(context, id)
         return self._make_bgpvpn_dict(bgpvpn_db, fields)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def update_bgpvpn(self, context, id, bgpvpn):
         bgpvpn_db = self._get_bgpvpn(context, id)
         if bgpvpn:
@@ -363,25 +361,25 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             bgpvpn_db.update(bgpvpn)
         return self._make_bgpvpn_dict(bgpvpn_db)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def delete_bgpvpn(self, context, id):
         bgpvpn_db = self._get_bgpvpn(context, id)
         bgpvpn = self._make_bgpvpn_dict(bgpvpn_db)
         context.session.delete(bgpvpn_db)
         return bgpvpn
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _make_net_assoc_dict(self, net_assoc_db, fields=None):
         res = {'id': net_assoc_db['id'],
                'tenant_id': net_assoc_db['tenant_id'],
                'bgpvpn_id': net_assoc_db['bgpvpn_id'],
                'network_id': net_assoc_db['network_id']}
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _get_net_assoc(self, context, assoc_id, bgpvpn_id):
         try:
-            query = self._model_query(context, BGPVPNNetAssociation)
+            query = model_query.query_with_hooks(context, BGPVPNNetAssociation)
             return query.filter(BGPVPNNetAssociation.id == assoc_id,
                                 BGPVPNNetAssociation.bgpvpn_id == bgpvpn_id
                                 ).one()
@@ -389,10 +387,10 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                     bgpvpn_id=bgpvpn_id)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def create_net_assoc(self, context, bgpvpn_id, net_assoc):
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 net_assoc_db = BGPVPNNetAssociation(
                     tenant_id=net_assoc['tenant_id'],
                     bgpvpn_id=bgpvpn_id,
@@ -407,21 +405,22 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             raise bgpvpn_ext.BGPVPNNetAssocAlreadyExists(
                 bgpvpn_id=bgpvpn_id, net_id=net_assoc['network_id'])
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_net_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
         net_assoc_db = self._get_net_assoc(context, assoc_id, bgpvpn_id)
         return self._make_net_assoc_dict(net_assoc_db, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_net_assocs(self, context, bgpvpn_id, filters=None, fields=None):
         if not filters:
             filters = {}
         filters['bgpvpn_id'] = [bgpvpn_id]
-        return self._get_collection(context, BGPVPNNetAssociation,
-                                    self._make_net_assoc_dict,
-                                    filters, fields)
+        return model_query.get_collection(
+            context, BGPVPNNetAssociation,
+            self._make_net_assoc_dict,
+            filters, fields)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def delete_net_assoc(self, context, assoc_id, bgpvpn_id):
         LOG.info("deleting network association %(id)s for "
                  "BGPVPN %(bgpvpn)s", {'id': assoc_id,
@@ -431,7 +430,7 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
         context.session.delete(net_assoc_db)
         return net_assoc
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _make_router_assoc_dict(self, router_assoc_db, fields=None):
         res = {'id': router_assoc_db['id'],
                'tenant_id': router_assoc_db['tenant_id'],
@@ -440,12 +439,13 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
                'advertise_extra_routes': router_assoc_db[
                    'advertise_extra_routes']
                }
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _get_router_assoc(self, context, assoc_id, bgpvpn_id):
         try:
-            query = self._model_query(context, BGPVPNRouterAssociation)
+            query = model_query.query_with_hooks(
+                context, BGPVPNRouterAssociation)
             return query.filter(BGPVPNRouterAssociation.id == assoc_id,
                                 BGPVPNRouterAssociation.bgpvpn_id == bgpvpn_id
                                 ).one()
@@ -453,11 +453,11 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             raise bgpvpn_ext.BGPVPNRouterAssocNotFound(id=assoc_id,
                                                        bgpvpn_id=bgpvpn_id)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def create_router_assoc(self, context, bgpvpn_id, router_association):
         router_id = router_association['router_id']
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 router_assoc_db = BGPVPNRouterAssociation(
                     tenant_id=router_association['tenant_id'],
                     bgpvpn_id=bgpvpn_id,
@@ -472,28 +472,29 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             raise bgpvpn_ext.BGPVPNRouterAssocAlreadyExists(
                 bgpvpn_id=bgpvpn_id, router_id=router_association['router_id'])
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_router_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
         router_assoc_db = self._get_router_assoc(context, assoc_id, bgpvpn_id)
         return self._make_router_assoc_dict(router_assoc_db, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_router_assocs(self, context, bgpvpn_id, filters=None, fields=None):
         if not filters:
             filters = {}
         filters['bgpvpn_id'] = [bgpvpn_id]
-        return self._get_collection(context, BGPVPNRouterAssociation,
-                                    self._make_router_assoc_dict,
-                                    filters, fields)
+        return model_query.get_collection(
+            context, BGPVPNRouterAssociation,
+            self._make_router_assoc_dict,
+            filters, fields)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def update_router_assoc(self, context, assoc_id, bgpvpn_id, router_assoc):
         router_assoc_db = self._get_router_assoc(context,
                                                  assoc_id, bgpvpn_id)
         router_assoc_db.update(router_assoc)
         return self._make_router_assoc_dict(router_assoc_db)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def delete_router_assoc(self, context, assoc_id, bgpvpn_id):
         LOG.info("deleting router association %(id)s for "
                  "BGPVPN %(bgpvpn)s",
@@ -504,7 +505,7 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
         context.session.delete(router_assoc_db)
         return router_assoc
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _make_port_assoc_dict(self, port_assoc_db, fields=None):
         routes = [port_assoc_route_dict_from_db(r)
                   for r in port_assoc_db['routes']]
@@ -514,12 +515,13 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
                'port_id': port_assoc_db['port_id'],
                'advertise_fixed_ips': port_assoc_db['advertise_fixed_ips'],
                'routes': routes}
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def _get_port_assoc(self, context, assoc_id, bgpvpn_id):
         try:
-            query = self._model_query(context, BGPVPNPortAssociation)
+            query = model_query.query_with_hooks(
+                context, BGPVPNPortAssociation)
             return query.filter(BGPVPNPortAssociation.id == assoc_id,
                                 BGPVPNPortAssociation.bgpvpn_id == bgpvpn_id
                                 ).one()
@@ -527,12 +529,12 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             raise bgpvpn_rc_ext.BGPVPNPortAssocNotFound(id=assoc_id,
                                                         bgpvpn_id=bgpvpn_id)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def create_port_assoc(self, context, bgpvpn_id, port_association):
         port_id = port_association['port_id']
         advertise_fixed_ips = port_association['advertise_fixed_ips']
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 port_assoc_db = BGPVPNPortAssociation(
                     tenant_id=port_association['tenant_id'],
                     bgpvpn_id=bgpvpn_id,
@@ -547,29 +549,30 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
             raise bgpvpn_rc_ext.BGPVPNPortAssocAlreadyExists(
                 bgpvpn_id=bgpvpn_id, port_id=port_association['port_id'])
 
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             for route in port_association['routes']:
-                    _add_port_assoc_route_db_from_dict(context,
-                                                       route, port_assoc_db.id)
+                _add_port_assoc_route_db_from_dict(
+                    context, route, port_assoc_db.id)
         return self._make_port_assoc_dict(port_assoc_db)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_port_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
         port_assoc_db = self._get_port_assoc(context, assoc_id, bgpvpn_id)
         return self._make_port_assoc_dict(port_assoc_db, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def get_port_assocs(self, context, bgpvpn_id, filters=None, fields=None):
         if not filters:
             filters = {}
         filters['bgpvpn_id'] = [bgpvpn_id]
-        return self._get_collection(context, BGPVPNPortAssociation,
-                                    self._make_port_assoc_dict,
-                                    filters, fields)
+        return model_query.get_collection(
+            context, BGPVPNPortAssociation,
+            self._make_port_assoc_dict,
+            filters, fields)
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def update_port_assoc(self, context, assoc_id, bgpvpn_id, port_assoc):
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             port_assoc_db = self._get_port_assoc(context, assoc_id, bgpvpn_id)
             for route_db in port_assoc_db.routes:
                 context.session.delete(route_db)
@@ -579,7 +582,7 @@ class BGPVPNPluginDb(common_db_mixin.CommonDbMixin):
         context.session.refresh(port_assoc_db)
         return self._make_port_assoc_dict(port_assoc_db)
 
-    @db_api.context_manager.writer
+    @db_api.CONTEXT_WRITER
     def delete_port_assoc(self, context, assoc_id, bgpvpn_id):
         LOG.info(("deleting port association %(id)s for "
                   "BGPVPN %(bgpvpn)s"),
