@@ -16,7 +16,6 @@
 from sqlalchemy import orm
 from sqlalchemy import sql
 
-from neutron.db import api as db_api
 from neutron.db.models import l3
 from neutron.db import models_v2
 from neutron.debug import debug_agent
@@ -26,6 +25,7 @@ from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as const
+from neutron_lib.db import api as db_api
 
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
@@ -42,7 +42,7 @@ LOG = logging.getLogger(__name__)
 
 
 @log_helpers.log_method_call
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_network_info_for_port(context, port_id, network_id):
     """Get MAC, IP and Gateway IP addresses informations for a specific port"""
     try:
@@ -51,7 +51,9 @@ def get_network_info_for_port(context, port_id, network_id):
                           models_v2.IPAllocation.ip_address,
                           models_v2.Subnet.cidr,
                           models_v2.Subnet.gateway_ip).
-                    join(models_v2.IPAllocation).
+                    join(models_v2.IPAllocation,
+                         models_v2.IPAllocation.port_id ==
+                         models_v2.Port.id).
                     join(models_v2.Subnet,
                          models_v2.IPAllocation.subnet_id ==
                          models_v2.Subnet.id).
@@ -79,7 +81,7 @@ def get_network_info_for_port(context, port_id, network_id):
             'gateway_mac': gateway_mac[0] if gateway_mac else None}
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_gateway_mac(context, network_id):
     gateway_mac = (
         context.session.
@@ -95,7 +97,7 @@ def get_gateway_mac(context, network_id):
     return gateway_mac[0] if gateway_mac else None
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_network_ports(context, network_id):
     # NOTE(tmorin): currents callers don't look at detailed results
     # but only test if at least one result exist => can be optimized
@@ -105,7 +107,7 @@ def get_network_ports(context, network_id):
                    models_v2.Port.admin_state_up == sql.true()).all())
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_router_ports(context, router_id):
     return (
         context.session.query(models_v2.Port).
@@ -116,7 +118,7 @@ def get_router_ports(context, router_id):
     )
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_router_bgpvpn_assocs(context, router_id):
     return (
         context.session.query(bgpvpn_db.BGPVPNRouterAssociation).
@@ -126,7 +128,7 @@ def get_router_bgpvpn_assocs(context, router_id):
     )
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_network_bgpvpn_assocs(context, net_id):
     return (
         context.session.query(bgpvpn_db.BGPVPNNetAssociation).
@@ -136,7 +138,7 @@ def get_network_bgpvpn_assocs(context, net_id):
     )
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_bgpvpns_of_router_assocs_by_network(context, net_id):
     return (
         context.session.query(bgpvpn_db.BGPVPN).
@@ -150,11 +152,11 @@ def get_bgpvpns_of_router_assocs_by_network(context, net_id):
     )
 
 
-@db_api.context_manager.reader
+@db_api.CONTEXT_READER
 def get_networks_for_router(context, router_id):
     ports = get_router_ports(context, router_id)
     if ports:
-        return set([port['network_id'] for port in ports])
+        return {port['network_id'] for port in ports}
     else:
         return []
 
@@ -326,7 +328,7 @@ class BaGPipeBGPVPNDriver(v2.BaGPipeBGPVPNDriver):
 
         return bgpvpn_network_info
 
-    @db_api.context_manager.reader
+    @db_api.CONTEXT_READER
     def retrieve_bgpvpns_of_router_assocs_by_network(self, context,
                                                      network_id):
         return [self.bgpvpn_db._make_bgpvpn_dict(bgpvpn) for bgpvpn in
@@ -378,8 +380,9 @@ class BaGPipeBGPVPNDriver(v2.BaGPipeBGPVPNDriver):
             self.agent_rpc.delete_bgpvpn(context, formated_bgpvpn)
 
     def _ignore_port(self, context, port):
-        if (port['device_owner'].startswith(const.DEVICE_OWNER_NETWORK_PREFIX)
-                and not port['device_owner'] in
+        if (port['device_owner'].startswith(
+                const.DEVICE_OWNER_NETWORK_PREFIX) and not
+            port['device_owner'] in
                 (debug_agent.DEVICE_OWNER_COMPUTE_PROBE,
                  debug_agent.DEVICE_OWNER_NETWORK_PROBE)):
             LOG.info("Port %s owner is network:*, we'll do nothing",
@@ -426,7 +429,7 @@ class BaGPipeBGPVPNDriver(v2.BaGPipeBGPVPNDriver):
                 pass
 
         elif (port['status'] == const.PORT_STATUS_DOWN and
-                original_port['status'] != const.PORT_STATUS_DOWN):
+              original_port['status'] != const.PORT_STATUS_DOWN):
             LOG.debug("notify_port_updated, port became DOWN")
             self.agent_rpc.detach_port_from_bgpvpn(context,
                                                    port_bgpvpn_info,
